import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

public class Dca extends Applet implements ActionListener
{

	public static int WIDTH=50,HEIGHT=50, RESOLUTION=10;
	public static double P = 0.5;

	Timer t;
	boolean paused;
	Button startButton, resetButton, pertButton;
	TextField width, height, prob;
	int[][] cells;

    public void init()
    {
		setSize(WIDTH*RESOLUTION+1, HEIGHT*RESOLUTION + 25);
		setLayout(new BorderLayout());
		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new GridLayout(1,2));
		add("South", buttonPane);

		startButton = new Button("Start");
		startButton.addActionListener(this);
		buttonPane.add(startButton);
		resetButton = new Button("Reset");
		resetButton.addActionListener(this);
		buttonPane.add(resetButton);
		prob = new TextField("0.5", 3);
		prob.addActionListener(this);
		buttonPane.add(prob);
		pertButton = new Button("Perturb");
		pertButton.addActionListener(this);
		buttonPane.add(pertButton);

		width = new TextField("50", 2);
		width.addActionListener(this);
		buttonPane.add(width);
		height = new TextField("50", 2);
		height.addActionListener(this);
		buttonPane.add(height);

    }

    public void start()
    {
		paused = true;
		reset();
		t = new Timer(200, new ActionListener() {
			public void actionPerformed(ActionEvent e){
				tick();
		}});
		t.start();
    }

    public void dosize()
    {
		try
		{
			WIDTH = Integer.parseInt(width.getText());
		}
		catch (NumberFormatException e)
		{
			WIDTH = 50;
		}
		if (WIDTH<=2) WIDTH=50;
		if (WIDTH>50) WIDTH=50;
		try
		{
			HEIGHT = Integer.parseInt(height.getText());
		}
		catch (NumberFormatException e)
		{
			HEIGHT = 50;
		}
		if (HEIGHT<=2) HEIGHT=50;
		if (HEIGHT>50) HEIGHT=50;
		resize(Math.max(WIDTH*RESOLUTION+1, 251), HEIGHT*RESOLUTION + 25);
	}

	public void doprob()
	{
		try
		{
			P = Double.parseDouble(prob.getText());
		}
		catch (NumberFormatException ex)
		{
			P = 0.5;
		}
	}

	public void reset()
	{
		dosize();
		doprob();
		cells=new int[WIDTH][HEIGHT];
		/*for(int i=0; i<cells.length;i++)
			for(int j=0; j<cells[0].length;j++)
				cells[i][j]=(int)(Math.random()*4);*/
		cells[WIDTH/2-1][HEIGHT/2]=1;
		cells[WIDTH/2+1][HEIGHT/2]=1;
		cells[WIDTH/2][HEIGHT/2-1]=1;
		cells[WIDTH/2][HEIGHT/2+1]=1;
		repaint();
	}

    public void destroy()
    {
		//
    }

    public void stop() {
		//
    }

    public String getAppletInfo()
    {
        return "Dissipative Cellular Automaton";
    }

	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == startButton)
		{
			if (paused)
			{
				paused = false;
				startButton.setLabel("Pause");
			} else {
				paused = true;
				startButton.setLabel("Start");
			}
		}
		else if (e.getSource() == resetButton)
		{
			reset();
			paused = true;
			startButton.setLabel("Start");
		}
		else if (e.getSource() == prob)
			doprob();
		else if (e.getSource() == pertButton)
			perturb();
	}

	public void paint(Graphics go) {
		Image im=new BufferedImage(WIDTH*RESOLUTION+2,HEIGHT*RESOLUTION+2,BufferedImage.TYPE_INT_RGB);
		Graphics g=im.getGraphics();
		g.setColor(Color.BLACK);
		g.fillRect(0,0,WIDTH*RESOLUTION+2,HEIGHT*RESOLUTION+2);
		for (int i=0;i<cells.length;i++)
		{
			for (int j=0;j<cells[0].length;j++)
			{
				int jr=j*RESOLUTION+1, ir=i*RESOLUTION+1;
				switch(cells[i][j])
				{
					case 0:
						g.setColor(Color.WHITE);
						break;
					case 1:
						g.setColor(Color.BLACK);
						break;
				}
				g.fillRect(ir,jr,RESOLUTION,RESOLUTION);
				g.setColor(Color.WHITE);
				g.drawLine(ir+RESOLUTION-1, jr, ir+RESOLUTION-1, jr+RESOLUTION-1);
				g.drawLine(ir, jr+RESOLUTION-1, ir+RESOLUTION-1, jr+RESOLUTION-1);
			}
		}
		go.drawImage(im,0,0,null);
	}
	public void update(Graphics go) {
		paint(go);
	}

	public void tick()
	{
		if (paused)
			return;
		int[][] new_cells=new int[WIDTH][HEIGHT];
		for (int i=0; i<WIDTH; ++i)
			for (int j=0; j<HEIGHT; ++j)
			{
				int[] nbd = new int[8];
				nbd[0] = (i>0)        ? (cells[i-1][j]) : cells[WIDTH-1][j];
				nbd[1] = (j>0)        ? (cells[i][j-1]) : cells[i][HEIGHT-1];
				nbd[2] = (i<WIDTH-1)  ? (cells[i+1][j]) : cells[0][j];
				nbd[3] = (j<HEIGHT-1) ? (cells[i][j+1]) : cells[i][0];
				nbd[4] = (i>0) ? (j>0) ? (cells[i-1][j-1]) : cells[i-1][HEIGHT-1] : (j>0) ? cells[WIDTH-1][j-1] : cells[WIDTH-1][HEIGHT-1];
				nbd[5] = (i<WIDTH-1) ? (j>0) ? (cells[i+1][j-1]) : cells[i+1][HEIGHT-1] : (j>0) ? cells[0][j-1] : cells[0][HEIGHT-1];
				nbd[6] = (i<WIDTH-1) ? (j<HEIGHT-1) ? (cells[i+1][j+1]) : cells[i+1][0] : (j<HEIGHT-1) ? cells[0][j+1] : cells[0][0];
				nbd[7] = (i>0) ? (j<HEIGHT-1) ? (cells[i-1][j+1]) : cells[i-1][0] : (j<HEIGHT-1) ? cells[WIDTH-1][j+1] : cells[WIDTH-1][0];
				if (Math.random()<P) {
					int count=0;
					for (int k=0; k<nbd.length; ++k)
						if (nbd[k]>0) ++count;
					if (cells[i][j] == 0)
					{
						if (count==2) new_cells[i][j]=1;
						else new_cells[i][j]=0;
					} else {
						if (count==1 || count==2) new_cells[i][j]=1;
						else new_cells[i][j]=0;
					}
				} else new_cells[i][j] = cells[i][j];
			}
		cells=new_cells;
		repaint();
	}

	public void perturb()
	{
		int x = (int)(Math.random()*WIDTH), y = (int)(Math.random()*HEIGHT);
		if (cells[x][y]>0) cells[x][y] = 0;
		else cells[x][y]=1;
		repaint();
	}

}
